/**
 * @file
 * A Backbone view for the edit_ui block element.
 */

(function (Drupal, Backbone, $, Modernizr) {
  "use strict";

  /**
   * Backbone view for the edit_ui block.
   */
  Drupal.editUi.block.BlockVisualView = Backbone.View.extend({
    /**
     * Custom data.
     */
    hiddenClass: 'visually-hidden',

    /**
     * Dom elements events.
     */
    events: function () {
      var events = {};
      if (Modernizr.touch) {
        events["touchstart"] = "startDrag";
        events["touchstart .contextual"] = function (e) {e.stopPropagation(); };
      }
      else {
        events["mousedown"] = "startDrag";
        events["mousedown .contextual"] = function (e) {e.stopPropagation(); };
      }
      return events;
    },

    /**
     * {@inheritdoc}
     */
    initialize: function (options) {
      // Add listeners.
      this.listenTo(this.model, 'change:isDragging', this.render);
      this.listenTo(this.model, 'change:content', this.render);
      this.listenTo(this.model, 'change:region', this.regionSaved);
      this.listenTo(this.model, 'destroy', this.remove);
      this.listenTo(this.model, 'calculateDimensions', this.calculateDimensions);

      // Init model state.
      this.model.setBlock(this.$el);
      this.model.show();
    },

    /**
     * {@inheritdoc}
     */
    render: function () {
      var content = this.model.get('content');
      this.$el.toggleClass(this.hiddenClass, this.model.get('isDragging'));

      if (content !== null) {
        if (content === '') {
          Drupal.detachBehaviors(this.$el.get(0));
          content = Drupal.theme('editUiBlockPlaceholderBlock', {
            id: this.model.get('id'),
            provider: this.model.get('provider'),
            label: this.model.get('label')
          });
        }

        // Insert new content.
        this.$el.html(content);
        Drupal.attachBehaviors(this.$el.get(0));
      }
    },

    /**
     * Fetch the model to get the content if block was previously disabled.
     */
    regionSaved: function () {
      if (this.model.previous('region') === '-1') {
        this.model.fetch();
      }
      else if (this.model.get('region') === '-1') {
        this.model.set(
          'content',
          Drupal.theme('editUiBlockDisabledBlock', this.model.attributes)
        );
      }
    },

    /**
     * Start drag.
     */
    startDrag: function (event) {
      var position;

      if (!Drupal.editUi.toolbar || !Drupal.editUi.toolbar.models.toolbarModel.get('isOpen')) {
        return;
      }
      if (event.type === 'mousedown' && event.button !== 0) {
        return;
      }
      event.preventDefault();

      // Calculate all dimensions.
      Drupal.editUi.utils.calculateDimensions();

      // Update model state.
      this.model.startDrag();

      // Re-calculate region dimensions (margin calculation correction).
      Drupal.editUi.region.collections.regionCollection.forEach(function (region) {
        region.trigger('calculateDimensions');
      });

      // Force block sort.
      Drupal.editUi.block.collections.blockCollection.sort();

      // Trigger events.
      position = Drupal.editUi.utils.getPosition(event);
      this.$el.trigger('startDrag', [
        this.model,
        position,
        this.model.get('dimensions'),
        this.model.get('offset')
      ]);
    },

    /**
     * Calculate the region dimensions.
     */
    calculateDimensions: function () {
      var offset = this.$el.offset();

      // Save dimensions.
      this.model.setOffset({
        top: offset.top,
        left: offset.left
      });
      this.model.setDimensions({
        width: this.$el.outerWidth(),
        height: this.$el.outerHeight()
      });
    }
  });

}(Drupal, Backbone, jQuery, Modernizr));
